import { getImageCaptcha, login, otpLogin } from '@/services/ant-design-pro/login';
import { CreditCardOutlined, LockOutlined, UserOutlined } from '@ant-design/icons';
import {
  LoginFormPage,
  ProConfigProvider,
  ProFormCheckbox,
  ProFormInstance,
  ProFormText,
} from '@ant-design/pro-components';
import { FormattedMessage, history, useIntl, useModel } from '@umijs/max';
import { message, Spin, Tabs } from 'antd';
import { createStyles } from 'antd-style';
import React, { useEffect, useRef, useState } from 'react';
import { flushSync } from 'react-dom';

const useStyles = createStyles(({}) => {
  return {
    container: {
      display: 'flex',
      flexDirection: 'column',
      height: '100vh',
      overflow: 'auto',
      backgroundImage:
        "url('https://mdn.alipayobjects.com/yuyan_qk0oxh/afts/img/V-_oS6r-i7wAAAAAAAAAAAAAFl94AQBr')",
      backgroundSize: '100% 100%',
    },
  };
});

const Login: React.FC = () => {
  const { initialState, setInitialState } = useModel('@@initialState');
  const intl = useIntl();
  const formRef = useRef<ProFormInstance>();
  const { styles } = useStyles();
  const [formLoading, setFormLoading] = useState(false);
  const [type, setType] = useState<string>('account');
  const [currentLoginStep, setCurrentLoginStep] = useState<string>('login');
  const [captchaUrl, setCaptchaUrl] = useState<string>('');
  const [captcha, setCaptcha] = useState<Captcha>();
  const [captchaSpinning, setCaptchaSpinning] = useState(false);
  /** 用户名密码登录token */
  const [uplToken, setUplToken] = useState<string>();

  /** 处理刷新验证码 */
  const handleRefreshCaptcha = async () => {
    setFormLoading(true);
    setCaptchaSpinning(true);
    await getImageCaptcha()
      .then((resp) => {
        if (resp.success) {
          setCaptcha(resp.data as Captcha);
          setCaptchaUrl('data:image/gif;base64,' + resp.data.img);
        }
      })
      .catch((err) => {
        console.log(err);
      });
    setCaptchaSpinning(false);
    setFormLoading(false);
  };

  /** 处理用户名密码登录 */
  const handleUsernameAndPasswordLogin = async () => {
    console.log(formRef.current);
    const param: API.LoginParams = {
      username: formRef.current?.getFieldValue('username'),
      password: formRef.current?.getFieldValue('password'),
      captcha: formRef.current?.getFieldValue('captcha'),
      uuid: captcha?.uuid,
    };
    await login(param)
      .then((resp) => {
        if (resp.success) {
          localStorage.setItem('Authorization', resp.data);
          setCurrentLoginStep('otpLogin');
          setUplToken(resp.data);
        }
      })
      .catch((err) => {
        console.log(err);
      });
  };

  /** 获取UserInfo */
  const fetchUserInfo = async () => {
    const userInfo = await initialState?.fetchUserInfo?.();
    if (userInfo) {
      flushSync(() => {
        setInitialState((s) => ({
          ...s,
          currentUser: userInfo,
        }));
      });
    }
  };

  /** 处理OTP登录 */
  const handleOtpLogin = async () => {
    if (captcha === undefined || captcha === null) {
      setCurrentLoginStep('login');
      return;
    }
    const param: API.otpLoginParam = {
      username: formRef.current?.getFieldValue('username'),
      code: formRef.current?.getFieldValue('otpCaptcha'),
      uplToken: uplToken,
    };
    return await otpLogin(param)
      .then((resp) => {
        if (resp.success) {
          localStorage.setItem('token', resp.data);
        } else {
          if (resp.errCode === '100007') {
            setCurrentLoginStep('login');
          }
        }
        return resp.success;
      })
      .catch((err) => {
        console.log(err);
        return false;
      });
  };

  /** 处理登录 */
  const handleFinish = async () => {
    setFormLoading(true);
    if (currentLoginStep === 'login') {
      await handleUsernameAndPasswordLogin();
    } else if (currentLoginStep === 'otpLogin') {
      const isSuccess = await handleOtpLogin();
      if (isSuccess) {
        await fetchUserInfo();
        const urlParams = new URL(window.location.href).searchParams;
        history.push(urlParams.get('redirect') || '/');
      }
    } else {
      message.error('不支持登录方式!');
    }
    setFormLoading(false);
  };

  /** 挂载处理逻辑 */
  useEffect(() => {
    handleRefreshCaptcha();
  }, []);

  return (
    <ProConfigProvider dark>
      <div className={styles.container}>
        <div
          style={{
            flex: '1',
          }}
        >
          <LoginFormPage
            backgroundVideoUrl="https://gw.alipayobjects.com/v/huamei_gcee1x/afts/video/jXRBRK_VAwoAAAAAAAAAAAAAK4eUAQBr"
            title="开放平台管理系统"
            containerStyle={{
              backgroundColor: 'rgba(0, 0, 0,0.65)',
              backdropFilter: 'blur(4px)',
            }}
            subTitle="惠融优贷金融云平台"
            onFinish={() => {
              handleFinish();
            }}
            formRef={formRef}
            loading={formLoading}
          >
            <Tabs
              activeKey={type}
              onChange={setType}
              centered
              items={[
                {
                  key: 'account',
                  label: intl.formatMessage({
                    id: 'pages.login.accountLogin.tab',
                    defaultMessage: '账户密码登录',
                  }),
                },
              ]}
            />

            {currentLoginStep === 'login' && (
              <>
                <ProFormText
                  name="username"
                  fieldProps={{
                    size: 'large',
                    prefix: <UserOutlined style={{ marginRight: '5px' }} />,
                  }}
                  placeholder={intl.formatMessage({
                    id: 'pages.login.username.placeholder',
                    defaultMessage: '用户名: admin or user',
                  })}
                  rules={[
                    {
                      required: true,
                      message: (
                        <FormattedMessage
                          id="pages.login.username.required"
                          defaultMessage="请输入用户名!"
                        />
                      ),
                    },
                  ]}
                />
                <ProFormText.Password
                  name="password"
                  fieldProps={{
                    size: 'large',
                    prefix: <LockOutlined style={{ marginRight: '5px' }} />,
                  }}
                  placeholder={intl.formatMessage({
                    id: 'pages.login.password.placeholder',
                    defaultMessage: '密码: ant.design',
                  })}
                  rules={[
                    {
                      required: true,
                      message: (
                        <FormattedMessage
                          id="pages.login.password.required"
                          defaultMessage="请输入密码！"
                        />
                      ),
                    },
                  ]}
                />

                <div>
                  <ProFormText
                    fieldProps={{
                      size: 'large',
                      addonAfter: (
                        <Spin spinning={captchaSpinning}>
                          <img
                            src={captchaUrl}
                            onClick={handleRefreshCaptcha}
                            style={{ minWidth: '90px' }}
                          />
                        </Spin>
                      ),
                      prefix: <CreditCardOutlined style={{ marginRight: '5px' }} />,
                    }}
                    name="captcha"
                    placeholder="请输入验证码"
                    rules={[
                      {
                        required: true,
                        message: '请输入验证码！',
                      },
                    ]}
                  />
                </div>
                <div
                  style={{
                    marginBottom: 24,
                  }}
                >
                  <ProFormCheckbox noStyle name="autoLogin">
                    <FormattedMessage id="pages.login.rememberMe" defaultMessage="自动登录" />
                  </ProFormCheckbox>
                  <a
                    style={{
                      float: 'right',
                    }}
                  >
                    <FormattedMessage id="pages.login.forgotPassword" defaultMessage="忘记密码" />
                  </a>
                </div>
              </>
            )}

            {currentLoginStep === 'otpLogin' && (
              <>
                <ProFormText
                  fieldProps={{
                    size: 'large',
                    prefix: <CreditCardOutlined style={{ marginRight: '5px' }} />,
                  }}
                  name="otpCaptcha"
                  placeholder="请输入OTP验证码"
                  required
                />
              </>
            )}
          </LoginFormPage>
        </div>
      </div>
    </ProConfigProvider>
  );
};

export default Login;
