import React, { useRef, useState } from 'react';
import { history } from 'umi';
import { PageContainer } from '@ant-design/pro-layout';
import { ProTable } from '@ant-design/pro-components';

import type { ProColumns, ActionType } from '@ant-design/pro-components';
import UpsertApiModalForm from './component/UpsertApiModalForm';
import { Button, message, Popconfirm, Spin, Tag } from 'antd';
import { PlusOutlined } from '@ant-design/icons';
import { deleteEgressApi, pageQueryEgressApi } from '@/services/op-admin/gateway/egress';
/**
 * @author bianlanzhou
 * @since 2024-09-23
 * @desc API列表页面
 */
const ApiTable: React.FC = () => {
  /** 引用 */
  const actionRef = useRef<ActionType>();

  const [deleteDomLoading, setDeleteDomLoading] = useState<boolean>(false);

  /** 处理查看 */
  const handleView = (data: EgressApiItem) => {
    history.push(
      {
        pathname: '/ui/gateway/egress/api/dtl',
      },
      data,
    )
  }

  /** 列定义 */
  const columns: ProColumns<EgressApiItem>[] = [
    {
      title: '序号',
      dataIndex: 'index',
      valueType: 'indexBorder',
      width: 48,
    },
    {
      title: 'API名称',
      dataIndex: 'apiName',
    },
    {
      title: 'API编码',
      dataIndex: 'apiCode',
    },
    {
      title: '创建人',
      dataIndex: 'createUsername',
      search: false,
    },
    {
      title: '创建时间',
      dataIndex: 'gmtCreate',
      search: false,
    },

    {
      title: '技术负责人',
      dataIndex: 'techOwnerUsername',
      search: false,
    },
    {
      title: '状态',
      dataIndex: 'status',
      search: false,
      render: (_, record) => {
        if (record.status === 'draft') {
          return [<Tag key='status' color='orange'>草稿</Tag>]
        }
        if (record.status === 'ok') {
          return [<Tag key='status' color='blue'>已启用</Tag>]
        }
        if (record.status === 'draft') {
          return [<Tag key='status' color='red'>已停用</Tag>]
        }
        if (record.status === 'delete') {
          return [<Tag key='status' color='purple'>已删除</Tag>]
        }
        return [<Tag key='status' color='#f50'>未知</Tag>]
      }
    },
    {
      title: '操作',
      valueType: 'option',
      width: '10%',
      render: (_, record) => [
        <a
          key="view"
          onClick={() => {
            handleView(record);
          }}
        >
          查看
        </a>,
        <UpsertApiModalForm
          key="edit"
          data={record}
          trigger={
            <a
              key="editable"
            >
              编辑
            </a>
          }
          callback={() => {
            actionRef.current?.reload();
          }}
        />,
        <Popconfirm
          key="delete"
          title="确认删除此条记录?"
          okText="确定"
          cancelText="取消"
          onConfirm={async () => {
            setDeleteDomLoading(true);
            await deleteEgressApi({ id: record.id }).then(() => {
              message.info("操作成功!")
              actionRef.current?.reload()
            })
            setDeleteDomLoading(false);
          }}
        >
          <Spin spinning={deleteDomLoading} >
            <a
              key="delete"
            >
              删除
            </a>
          </Spin>
        </Popconfirm>,
      ],
    },
  ]
  return (
    <PageContainer
      header={{
        title: '',
      }}
    >
      <ProTable<EgressApiItem>
        columns={columns}
        cardBordered
        actionRef={actionRef}
        request={pageQueryEgressApi}
        editable={{
          type: 'multiple',
        }}
        rowKey={'id'}
        search={{
          labelWidth: 'auto',
        }}
        pagination={{
          pageSize: 10,
        }}
        toolBarRender={() => [
          <UpsertApiModalForm
            key="create"
            trigger={
              <Button key="create" type="primary" onClick={() => { }} icon={<PlusOutlined />}>
                创建API
              </Button>
            }
            callback={() => {
              actionRef.current?.reload();
            }}
          />,
        ]}
        headerTitle={'接出API列表'}
      />
    </PageContainer>
  );
}
export default ApiTable;